﻿using System;
using Hello1.CommandStack.Events;
using Memento.Domain;

namespace BankDemo.Cqrs.CommandStack.Aggregates
{
    public class AccountRequest : Aggregate,
        IApplyEvent<AccountRequestReceivedEvent> 
    {
        public Guid RequestId { get; set; }
        public DateTime Received { get; set; }
        public string CustomerName { get; set; }
        public int Age { get; set; }
        public bool IsNew { get; set; }

        public void ApplyEvent(
            [AggregateId("RequestId")] AccountRequestReceivedEvent theEvent)
        {
            RequestId = theEvent.RequestId;
            Received = DateTime.UtcNow;
            Age = theEvent.Age;
            IsNew = theEvent.IsNewCustomer;
            CustomerName = theEvent.FullName;
        }

        public static class Factory
        {
            public static AccountRequest NewRequestFrom(string name, int age, bool isNew)
            {
                var received = new AccountRequestReceivedEvent(Guid.NewGuid(), name, age, isNew);

                // Tell the aggregate to log the "received" event
                var request = new AccountRequest();
                request.RaiseEvent(received);
                return request;
            }
        }
    }
}