﻿using BankDemo.Application;
using BankDemo.Cqrs.CommandStack.Aggregates;
using BankDemo.Cqrs.CommandStack.Commands;
using BankDemo.Cqrs.CommandStack.Events;
using Memento.Messaging.Postie;
using Memento.Persistence;

namespace BankDemo.Cqrs.CommandStack.Sagas
{
    public class AccountRequestSaga : Saga,
        IAmStartedBy<RequestNewBankAccountCommand>,
        IHandleMessages<BankAccountApprovedEvent>
    {
        public AccountRequestSaga(IBus bus, IEventStore eventStore, IRepository repository)
            : base(bus, eventStore, repository)
        {
        }

        public void Handle(RequestNewBankAccountCommand message)
        {
            var request = AccountRequest.Factory.NewRequestFrom(
                message.FullName, message.Age, message.IsNew);
            Repository.Save(request);

            // Notify back
            var hub = new AccountRequestHub();
            hub.NotifyReceived(message.SignalRConnectionId, request.RequestId);
        }

        public void Handle(BankAccountApprovedEvent message)
        {
            throw new System.NotImplementedException();
        }
    }
}