﻿using System.Web;
using System.Web.Mvc;
using System.Web.Routing;
using BankDemo.Cqrs.CommandStack.Sagas;
using BankDemo.Cqrs.QueryStack.Denormalizers;
using Memento.Messaging;
using Memento.Messaging.Postie;
using Memento.Messaging.Postie.Unity;
using Memento.Persistence;
using Memento.Persistence.EmbeddedRavenDB;
using Microsoft.Practices.Unity;
using Raven.Client.Embedded;
using Raven.Database.Server;

namespace BankDemo
{
    public class MvcApplication : HttpApplication
    {
        public static IBus Bus { get; private set; }
        public static IRepository AggregateRepository { get; private set; }

        protected void Application_Start()
        {
            AreaRegistration.RegisterAllAreas();
            RouteConfig.RegisterRoutes(RouteTable.Routes);

            // Initialize the event store (RAVENDB)
            NonAdminHttp.EnsureCanListenToWhenInNonAdminContext(8080);
            var documentStore = new EmbeddableDocumentStore
            {
                ConnectionStringName = "EventStore",
                UseEmbeddedHttpServer = true
            };
            documentStore.Configuration.Port = 8080;
            documentStore.Initialize();

            // Configure the FX
            var container = MementoStartup
                .UnityConfig<InMemoryBus, EmbeddedRavenDbEventStore, EmbeddableDocumentStore>(
                    documentStore);
            
            // Save global references to the FX core elements
            Bus = container.Resolve<IBus>();
            AggregateRepository = container.Resolve<IRepository>();

            // Add sagas and handlers to the bus
            Bus.RegisterSaga<AccountRequestSaga>();
            Bus.RegisterHandler<AccountRequestDenormalizer>();
        }
    }
}
